%{
#include <stack>
#include <string>
#include "syntax.h"
#include "parser.h"
using namespace std;

stack<int> s;

#define YY_USER_INIT s.push(0); BEGIN(freshline);

%}

%option noyywrap

%x freshline

digit	[0-9]
letter	[A-Za-z]

%%
<freshline>""/[^\t ]		{
if (s.top() == 0) {
   BEGIN(0);
} else {
  s.pop(); yyless(0); return DEDENT;
}
				}
<freshline>[ \t]+		{
if (s.top() == yyleng) {
  BEGIN(0); // Same indentation, continue
} else if (s.top() > yyleng) {
  s.pop(); yyless(0); return DEDENT;
  // Same rule again until the stack is even
} else {
  s.push(yyleng); BEGIN(0); return INDENT;
}
	       			}
#[^\n]*					// Eat comments
\"[^"]*\"				yylval.sym = new string(yytext); return STRING;
if					return IF;
else					return ELSE;
elif 					return ELIF;
{digit}+|{digit}*\.{digit}+		yylval.num = atof(yytext); return NUM;
{letter}({letter}|{digit}|[?.-])*	yylval.sym = new string(yytext); return ID;
"<-"					return GETS;
"->"					return MAPS;
".."					return CAT;
[(){}.,*/+-]				return yytext[0];
[\t ]					// Eat whitespace not first on a line
"/"\n					// Eat newlines ending in /
[\n;]					BEGIN(freshline); return STOP;
.					fprintf(stderr, "Scanning error!\nOffender: %s\n", yytext); exit(1);
%%